﻿/******************************************************************************/
/*!
NEEURO SENZEBAND SDK Version 1.0
\file NSB_EEG.cs
\author Ngoh Seng Sin Isaac
\date 18/11/2016
\brief
This file consists of calls to the NSB_EEG as well as functions that the plugins will do callBacks from

Public functions include:

	//Initialization functions

	//Initializes the EEG. Only call this after the Bluetooth has been initialized
	void EEG_Init()

	//Tells the app to start reading the EEG data sent by the NEEURO device
	void EEG_Start()

	//Tells the app to stop reading the EEG data sent by the NEEURO device
	void EEG_Stop()


	//Delegates. All delegates calls occur frequently. So any heavy calculation code shouldn't be put inside them

	//Delegate to check for the status of the 4 channels. 
	void assignChannelDelegate(EEGController.btBoolArrayCallback callBackFunc)

	//Delegate to take in the raw EEG signal.
	void assignRawDataDelegate(EEGController.btFloatCallback callBackFunc)

	//Delegate to take in the attention signal.
	void assignAttentionDelegate(EEGController.btFloatCallback callBackFunc)

	//Delegate to take in the relaxation signal.
	void assignRelaxationDelegate(EEGController.btFloatCallback callBackFunc)

	//Delegate to take in the mental WL signal.
	void assignMentalWorkloadDelegate(EEGController.btFloatArrayCallback callBackFunc)

	//Delegate to take in the ABDT signal.
	void assignABDTDelegate(EEGController.btFloatArrayCallback callBackFunc)

	//Delegate to take in the accelerometer.
	void assignAccDelegate(EEGController.btFloatArrayCallback callBackFunc)

	//Call this when you are going to change scenes or when the delegates you previously assigned will be null
	void unloadDelegates()


//*WARNING* DO NOT MODIFY ANY OF THE PRIVATE FUNCTIONS. 
Private functions include:

	void EEG_ChangeColorCode( string mess )
	void EEG_GetData ( string tmpdata )
	void EEG_GetAccXYZ ( string tmpdata )
	void EEG_GetAttention ( string tmpData )
	void EEG_GetFreqTypePower ( string tmpData )
	void EEG_GetMentalStateData ( string tmpdata )
	void EEG_GetSVM_Attention ( string tmpData )
	void EEG_GetSVM_MWorkload ( string tmpData )

*/
/******************************************************************************/

using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
#if UNITY_IOS
using NSB_SDK_IOS;
#elif UNITY_ANDROID
using NSB_SDK_ANDROID;
#endif

public class NSB_EEG : MonoBehaviour {

	public static NSB_EEG instance;

	public delegate void btValueCallback(float btValues);
	public delegate void stringReturnVoid(string btDetails); 
	//private btValueCallback btAttentionValueDelegate;

	#if UNITY_IOS
	private stringReturnVoid BLEBatteryUpdate; 				//Callback that will be invoked when the battery level is updated. This is called frequently.
	#endif

	void Awake()
	{
		if(instance != null)
			Destroy(this.gameObject);

		if(instance == null)
		{
			instance = this;
			initializeEEGDelegates();
		}

		DontDestroyOnLoad(this.gameObject);
	}

#region initialize EEG
/******************************************************************************/
/*!
\fn  void EEG_Init()
\brief Initializes the EEG. This is automatically called when the bluetooth has been initialized.
*/
/******************************************************************************/
	public void EEG_Init()
	{

		#if (!UNITY_EDITOR)

		initializeEEGDelegates ();

		#else

		Debug.Log("EEG_Init is called but you are in editor mode. ");

		#endif
	}

/******************************************************************************/
/*!
\fn  void EEG_Start ()
\brief Starts the EEG to grab the data from the NEEURO device
*/
/******************************************************************************/
	public void EEG_Start ()
	{
		EEGController.getInstance().EEG_StartReceiving();
	}

/******************************************************************************/
/*!
\fn  void EEG_Stop ()
\brief Stops the grabbing of data from the NEEURO device
*/
/******************************************************************************/
	public void EEG_Stop ()
	{
		EEGController.getInstance().EEG_StopReceiving();
	}

#endregion

#region assignDelegates
/******************************************************************************/
/*!
\fn  void assignChannelDelegate(EEGController.btBoolArrayCallback callBackFunc)
\brief The assigned delegate will be called each time the 4 channels update.
\param callBackFunc
The callBack function to execute
*/
/******************************************************************************/
	public void assignChannelDelegate(EEGController.btBoolArrayCallback callBackFunc)
	{
		EEGController.getInstance().assignChannelStatusDelegate(callBackFunc);
	}


/******************************************************************************/
/*!
\fn  void assignGoodConnectionCheckDelegate(EEGController.btBoolCallback callBackFunc)
\brief The assigned delegate will be called whenever the NEEURO device updates the connection checks. 
\param callBackFunc
The callBack function to execute
*/
/******************************************************************************/
	public void assignGoodConnectionCheckDelegate(EEGController.btBoolCallback callBackFunc)
	{
		EEGController.getInstance().assignGoodConnectionCheckDelegate(callBackFunc);
	}

/******************************************************************************/
/*!
\fn  void assignRawDataDelegate(MyEEGHandler.btFloatArrayCallback callBackFunc)
\brief The assigned delegate will be called when the raw EEG signal is ready. The raw signal has is an array of 1000 floats.
	   The first 250 floats belong to one channel, next 250 floats belong to another channel and so on.
\param callBackFunc
The callBack function to execute
*/
/******************************************************************************/
/*	public void assignRawDataDelegate(EEGController.btFloatArrayCallback callBackFunc)
	{
		EEGController.getInstance().assignRawDataDelegate(callBackFunc);
	}
*/
/******************************************************************************/
/*!
	\fn  void assignAttentionDelegate(MyEEGHandler.btFloatCallback callBackFunc)
\brief The assigned delegate will be called when the attention signal is ready.
\param callBackFunc
The callBack function to execute
*/
/******************************************************************************/
	public void assignAttentionDelegate(EEGController.btFloatCallback callBackFunc)
	{
		EEGController.getInstance().assignAttentionDelegate(callBackFunc);
	}

/******************************************************************************/
/*!
	\fn  void assignRelaxationDelegate(MyEEGHandler.btFloatCallback callBackFunc)
\brief The assigned delegate will be called when the relaxation signal is ready. This is an array of 4 floats
\param callBackFunc
The callBack function to execute
*/
/******************************************************************************/
	public void assignRelaxationDelegate(EEGController.btFloatCallback callBackFunc)
	{
		EEGController.getInstance().assignRelaxationDelegate(callBackFunc);
	}

/******************************************************************************/
/*!
	\fn  void assignMentalWorkloadDelegate(MyEEGHandler.btFloatCallback callBackFunc)
\brief The assigned delegate will be called when the attention signal is ready. This is an array of 4 floats
\param callBackFunc
The callBack function to execute
*/
/******************************************************************************/
/*	public void assignMentalWorkloadDelegate(EEGController.btFloatCallback callBackFunc)
	{
		EEGController.getInstance().assignMentalWorkloadDelegate(callBackFunc);
	}
*/

/******************************************************************************/
/*!
\fn  void assignABDTDelegate(MyEEGHandler.btFloatArrayCallback callBackFunc)
\brief The assigned delegate will be called when the ABDT(Alpha, Beta, Delta..) signal is ready. This is a 2D array of 4x5 floats
\param callBackFunc
The callBack function to execute
*/
/******************************************************************************/
/*	public void assignABDTDelegate(EEGController.btFloat2DArrayCallback callBackFunc)
	{
		EEGController.getInstance().assignABDTDeletgate(callBackFunc);
	}
*/
/******************************************************************************/
/*!
\fn  void assignAccDelegate(MyEEGHandler.btFloatArrayCallback callBackFunc)
\brief The assigned delegate will be called when accelerometer updates. This is an array of 3 floats
\param callBackFunc
The callBack function to execute
*/
/******************************************************************************/
	public void assignAccDelegate(EEGController.btFloatArrayCallback callBackFunc)
	{
		EEGController.getInstance().assignACCDelegate(callBackFunc);
	}

/******************************************************************************/
/*!
\fn void getBatteryStatus(BLEConnectedCallBack batteryStatusCallBack)
\brief  Gets the battery status of the NEEURO device. The callBack will be called as frequent as the EEG data.
\param batteryStatusCallBack
CallBack to return the battery status once its ready.
*/
/******************************************************************************/
	public void assignBatteryStatus(EEGController.EEGStringCallBack batteryStatusCallBack)
	{
		EEGController.getInstance().assignBatteryUpdateCallBack(batteryStatusCallBack);
	}

/******************************************************************************/
/*!
\fn  void unloadDelegates()
\brief Call this when you are going to change scenes or when the delegates you previously assigned will be null
*/
/******************************************************************************/
	public void unloadDelegates()
	{
//		EEGController.getInstance().assignRawDataDelegate(null);
		EEGController.getInstance().assignAttentionDelegate(null);	
		EEGController.getInstance().assignRelaxationDelegate(null);
//		EEGController.getInstance().assignMentalWorkloadDelegate(null);
//		EEGController.getInstance().assignABDTDeletgate(null);
		EEGController.getInstance().assignACCDelegate(null);
		EEGController.getInstance().assignChannelStatusDelegate(null);
		EEGController.getInstance().assignGoodConnectionCheckDelegate(null);
	}

#endregion

#region private functions. These are used by the plugins itself

/*	private void EEG_GetData ( string tmpdata )
	{
		EEGController.getInstance().EEG_GetData(tmpdata);
	}	
*/
	private void EEG_GetAccXYZ ( string tmpdata )
	{
		EEGController.getInstance().EEG_GetAccXYZ(tmpdata);
	}

/*	private void EEG_GetFreqTypePower ( string tmpData )
	{
		EEGController.getInstance().EEG_GetFreqTypePower(tmpData);
	}
*/
	private void EEG_GetMentalStateData ( string tmpdata )
	{
		EEGController.getInstance().EEG_GetMentalStateData(tmpdata);
	}

	private void EEG_GetChannelStatus ( string tmpData )
	{
		EEGController.getInstance().EEG_GetChannelStatus(tmpData);
	}

	private void EEG_GetGoodConnectionCheck ( string tmpData )
	{
		EEGController.getInstance().EEG_GetGoodConnectionCheck(tmpData);
	}

	private void EEG_batteryLevelUpdate (string tmpData)
	{		
		EEGController.getInstance().EEG_GetBatteryLevelUpdate(tmpData);
	}




	private void initializeEEGDelegates()
	{
		#if UNITY_IOS && !UNITY_EDITOR
		Debug.Log ("initializeEEGDelegates start");
		EEGController.getInstance ().putEEG_GetAccXYZ (EEG_GetAccXYZ);
//		EEGController.getInstance ().putEEG_GetData (EEG_GetData);
//		EEGController.getInstance ().putEEG_GetFreqTypePower (EEG_GetFreqTypePower);
		EEGController.getInstance ().putEEG_GetMentalStateData (EEG_GetMentalStateData);
		EEGController.getInstance ().putEEG_batteryLevelUpdate (EEG_batteryLevelUpdate);
		EEGController.getInstance ().putEEG_GetGoodConnectionCheck (EEG_GetGoodConnectionCheck);
		EEGController.getInstance ().putEEG_GetChannelStatus (EEG_GetChannelStatus);

		Debug.Log ("initializeEEGDelegates end");
		#endif
	}

#endregion

}


