﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class NSB_Manager : MonoBehaviour {


/*
	NSBM Init
		- false
		- true 	- Ready

	Scanning
		- false - Not scanning
		- true 	- Scanning
		
	Connection States
		- 0 	- Not connected
		- 1 	- Connecting
		- 2		- Connected, connectionStatus is updated in DeviceStatus callback
		- 3		- Connected, SB address is returned in ConnectionSucceed callback
		- 4		- Connected, MCUID is received
		Note:
		iOS devices 0->1->3->4
		Android devices 0->1->2->3->4

	EEG Started
		- false
		- true
*/

//Information stored for SB
	public List<string> listAvailableDevices = new List<string> (); 	//list of available SB addresses, from scanning

	private bool 	bIsInitCompleted = false;							//state, if NSB systems are ready
	private bool 	bIsScanning = false;								//state, if scanning
	private bool 	bEegStarted = false;								//state, if EEG is being transmitted, received
	private int 	connectionState = 0;								//state, on the connection
	private int		prevConnectionState = 0;

	private string 	addressConnectingSB = string.Empty;					//holds address of currently connecting SB
	private string 	addressConnectedSB = string.Empty;					//holds address of currently connected SB
	private string 	mcuid = string.Empty;								//holds the MCUID when available (of connected SB)
	private string 	connectionStatus = string.Empty;					// "Not connected", "Connecting" or "Connected"  from NSB_BLE.getConnectingString(), getConnectedString(), getNotConnectedString
	private string 	bluetoothStatus = string.Empty;						//holds the OS's bluetooth status,  enabled or disabled
	private string 	batteryLevel = string.Empty;						//holds the battery level

	private float[] mentalStateData = new float[4];
	private int[] 	accelerometerData = new int[3];
	private bool[] 	channelStatus = new bool[4];
	private bool 	goodContact;



//UNITY mono functions
	// Use this for initialization
	void Start () {
		bIsInitCompleted = false;
		Screen.sleepTimeout = SleepTimeout.NeverSleep;

		Init ();
	}
	
	// Update is called once per frame
	void Update () {

		//Tracking connection state;
		if (connectionState != prevConnectionState) {
			Debug.Log ("NSB Connection state, "+prevConnectionState+" -> "+connectionState);
			prevConnectionState = connectionState;
		}

	}




//Public functions for controlling SB
	public void Init()
	{
		//Initialises Bluetooth, NSB libraries, and also sets the callback functions when hardwares' actions are completed.
		if (bIsInitCompleted == false) {



			//BLE - Bluetooth system controls
			NSB_BLE.instance.assignErrorLogDelegate (ErrorLog);
			NSB_BLE.instance.initializeBT (InitComplete, GetDeviceStatus, GetBTStatus);
			NSB_EEG.instance.assignBatteryStatus (GetBattery);
			NSB_BLE.instance.assignScanCallBack (FoundAvailableDevice);

			//EEG - EEG signal processing controls
			NSB_EEG.instance.assignAttentionDelegate (grabAttention);
			NSB_EEG.instance.assignRelaxationDelegate (grabRelaxation);
//			NSB_EEG.instance.assignMentalWorkloadDelegate (grabMentalWorkload);
			NSB_EEG.instance.assignAccDelegate (grabAccelerometer);
			NSB_EEG.instance.assignChannelDelegate (grabChannelStatus);
			NSB_EEG.instance.assignGoodConnectionCheckDelegate (grabGoodConnection);		
		}
	}

	public bool IsInitCompleted()
	{
		return bIsInitCompleted;
	}

	public bool IsBluetoothEnabled()
	{
		if(bluetoothStatus.Contains("off"))
			return false;
		else if(bluetoothStatus.Contains("on"))
			return true;
		return false;	//the state is not known
	}

	public bool IsScanning()
	{
		//returns if the app is scanning for available SB device or not
		return bIsScanning;
	}

	public void SetScanning(bool state)
	{
		//Controls the mobile/tablet to turn on or turn off scanning for SB device
		if (bIsScanning != state) {
			Debug.Log ("NSB SetScanning "+state);
			NSB_BLE.instance.startStopScanning (state);
			bIsScanning = state;
			if (!bIsScanning) {
				//Turn off scanning
				ClearList ();
				CancelInvoke ("ClearList");
			} else {
				//Turn on scanning
				InvokeRepeating("ClearList",6,6);
			}
		}		
	}

	public bool ConnectSB(string address)
	{
		//Starts connection process to SB device
		Debug.Log ("NSB ConnectSB "+address);
		if(NSB_BLE.instance.connectBT(address, ConnectionSucceed, ConnectionBroken, ConnectionFailed) == false) {
			//Unable to process this function
			Debug.Log ("NSB Can't start connect");
			return false;
		}

		addressConnectingSB = address;	//TEMP
		ClearList ();
		return true;		
	}

	public void DisconnectSB()
	{
		//Disconnects a connected SB device
		Debug.Log ("NSB DisconnectSB");
		if (addressConnectedSB != string.Empty) {
			NSB_BLE.instance.disconnectBT (addressConnectedSB);

		} else if (addressConnectingSB != string.Empty) {
			NSB_BLE.instance.disconnectBT (addressConnectingSB);
		}	
	}

	public int GetConnectionState()
	{
		/*
		Connection States
		- 0 	- Not connected
		- 1 	- Connecting
		- 2		- Connected, connectionStatus is updated in DeviceStatus callback
		- 3		- Connected, SB address is returned in ConnectionSucceed callback
		- 4		- Connected, MCUID is received
		*/
		connectionState = 0;

		if(connectionStatus == NSB_BLE.instance.getNotConnectedString ())
			connectionState = 0;
		else if(connectionStatus == NSB_BLE.instance.getConnectingString ())
			connectionState = 1;
		else if(connectionStatus == NSB_BLE.instance.getConnectedString ())
		{
			if(addressConnectedSB == string.Empty)
				connectionState = 2;
			else{
				if(mcuid == string.Empty)
					connectionState = 3;
				else
					connectionState = 4;
			}
		}
		return connectionState;
	}

	public string GetConnectedSBAddress()
	{
		return addressConnectedSB;
	}

	public string GetConnectedSBMCUID()
	{
		return mcuid;
	}

	public bool GetReceiveEEGState()
	{
		return bEegStarted;
	}

	public void SetReceiveEEG(bool send)
	{
		Debug.Log ("NSB Set receive EEG " + send); 
		if (send) {
			NSB_EEG.instance.EEG_Start ();
			bEegStarted = true;
		} else {
			NSB_EEG.instance.EEG_Stop ();
			bEegStarted = false;
		}	
	}

	//EEG & received signal related functions
	public int GetAccel(int dimension)
	{
		if(dimension < 3)
			return accelerometerData [dimension];
		else
			return 999;	//Error; dimension cover X, Y, Z only.
	}

	public float GetAttention()
	{
		return mentalStateData [1];
	}

	public float GetRelaxation()
	{
		return mentalStateData [0];
	}

/*	public float GetMentalWL()
	{
		return mentalStateData [2];
	}
*/
	public bool GetChannelStatus(int channel)
	{
		if (channel < 4)
			return channelStatus [channel];
		else
			return false;
	}

	public bool GetGoodContact()
	{
		return goodContact;
	}


//CALLBACKS to be sent to NSB_BLE and NSB_EEG
	void ErrorLog(string error)
	{
		//Error log
		Debug.LogError("NSB SDK ERROR LOG: " + error);
	}

	void InitComplete()
	{
		//reset all data
		bIsScanning = true;
		addressConnectingSB = string.Empty;
		addressConnectedSB = string.Empty;
		mcuid = string.Empty;
		connectionStatus = string.Empty;
		bEegStarted = false;
		connectionState = 0;

		ClearList ();
		bIsInitCompleted = true;
	}

	void GetDeviceStatus(string status)
	{
		//Updates the status of the connection of the SenzeBand device
		Debug.Log ("NSB SB connection status changed: "+status);
		connectionStatus = status;
		if (status == NSB_BLE.instance.getConnectedString ()) {
			//connected!		also see ConnectionSucceed()
		} else if (status == NSB_BLE.instance.getNotConnectedString ()) {
			//disconnected!		also see ConnectionFailed()
		} else if (status == NSB_BLE.instance.getConnectingString ()) {
			//connecting!
		}
	}

	void GetBTStatus(string status)
	{
		//Updates the status of Bluetooth settings
		Debug.Log ("NSB BT setting changed: "+status);
		bluetoothStatus = status;
	}

	void GetBattery(string battery)
	{
		//Updates the battery level
		batteryLevel = battery;
	}

	void FoundAvailableDevice(string address)
	{
		//During scanning, an available SenzeBand is found
		Debug.Log ("NSB found available device : "+address+" ; list of "+listAvailableDevices.Count);
		string tempString = string.Copy(address);
		listAvailableDevices.Add(tempString);
	}

	IEnumerator co_PullMCUID()
	{
		yield return new WaitForSeconds(1.0f);
		if (connectionStatus == NSB_BLE.instance.getConnectedString() && addressConnectedSB != string.Empty)
		{
			
			while (mcuid == string.Empty) {
				yield return new WaitForSeconds(0.5f);

				//Gets from NSB to store the MCU ID of the SB device
				mcuid = NSB_BLE.instance.getMCUID (addressConnectedSB);
				if (mcuid.Length < 8)
					mcuid = string.Empty;
				Debug.Log ("NSB Pull mcuid: "+mcuid);

				if (connectionStatus != NSB_BLE.instance.getConnectedString () || addressConnectedSB == string.Empty)
					break;
			}
		}
	}

	void ConnectionSucceed(string address)
	{
		//Callback for when the connection process is successful - SenzeBand is connected.
		Debug.Log ("NSB connection succeed : "+address);
		addressConnectedSB = address;
		addressConnectingSB = string.Empty;
		bEegStarted = false;
		StartCoroutine (co_PullMCUID());
		//NSB_EEG.instance.EEG_Start();

	}

	void ConnectionBroken(string address)
	{
		//Callback for when the connection process fails - SenzeBand is NOT connected.
		//OR after a successful connection, when the connection is broken
		Debug.Log ("NSB connection broken : "+address);
		addressConnectedSB = string.Empty;
		addressConnectingSB = string.Empty;
		mcuid = string.Empty;
		bEegStarted = false;
		ClearList ();
	}

	void ConnectionFailed(string address)
	{
		//Callback for when the connection process fails - SenzeBand is NOT connected.
		//OR after a successful connection, when the connection is broken
		Debug.Log ("NSB connection failed : "+address);
		addressConnectedSB = string.Empty;
		addressConnectingSB = string.Empty;
		mcuid = string.Empty;
		bEegStarted = false;
		ClearList ();
	}

	void grabAttention(float attention)
	{
		mentalStateData[1] = attention;
	}

	void grabRelaxation(float relaxation)
	{
		mentalStateData[0] = relaxation;
	}

	void grabMentalWorkload(float mentalWorkload)
	{
		mentalStateData[2] = mentalWorkload;
	}

	void grabAccelerometer(float[] acc)
	{
		if (acc.Length == 3) {
			accelerometerData [0] = (int)acc [0];
			accelerometerData [1] = (int)acc [1];
			accelerometerData [2] = (int)acc [2];
		} else
			Debug.Log ("NSB accelerometer has incorrect data");
	}

	void grabChannelStatus(bool[] chnStatus)
	{
		if (chnStatus.Length == 4) {
			channelStatus [0] = chnStatus [0];
			channelStatus [1] = chnStatus [1];
			channelStatus [2] = chnStatus [2];
			channelStatus [3] = chnStatus [3];
		}else
			Debug.Log ("NSB channelStatus has incorrect data");
	}

	void grabGoodConnection(bool goodConnection)
	{
		goodContact = goodConnection;
	}

//Other Internal Support functions

	void ClearList()
	{
		//Clearing the list of SenzeBands available for connection
		listAvailableDevices.Clear();
		NSB_BLE.instance.clearList ();
	}

}
